// Copyright (C) 2014 ChaosForge Ltd
// http://chaosforge.org/
//
// This file is part of NV Libraries.
// For conditions of distribution and use, see copyright notice in nv.hh

#ifndef NV_ASSIMP_LOADER_HH
#define NV_ASSIMP_LOADER_HH

#include <nv/common.hh>
#include <nv/interface/mesh_loader.hh>
#include <nv/interface/mesh_data.hh>

namespace nv 
{
	struct assimp_bone
	{
		string name;
		mat4   offset;

		assimp_bone(){}
		assimp_bone( const string& n, const mat4& m ) : name(n), offset(m) {}
	};

	struct assimp_model
	{
		std::vector< mesh_data* >  meshes;
		std::vector< assimp_bone > bones;
	};

	class assimp_loader : public mesh_loader
	{
	public:
		assimp_loader( const string& a_ext, const mat4& a_rotate_transform, float a_scale, uint32 a_assimp_flags ) 
			: m_ext( a_ext ), m_rotate_transform( a_rotate_transform ), m_scale( a_scale ), m_assimp_flags( a_assimp_flags ), m_mesh_count(0), m_scene( nullptr ) {}
		virtual bool load( stream& source );
		virtual mesh_data* release_mesh_data( size_t index = 0 );
		virtual size_t get_mesh_count() const { return m_mesh_count; }
		virtual ~assimp_loader();
		assimp_model* release_merged_model();
		bool load_bones( size_t index, std::vector< assimp_bone >& bones );
		void scene_report() const;
	private:

		string m_ext;
		mat4   m_rotate_transform;
		float  m_scale;
		uint32 m_assimp_flags;
		size_t m_mesh_count;
		const void* m_scene;
	};

	struct assimp_plain_vtx 
	{
		vec3 position;
		vec3 normal;
		vec2 texcoord;
		vec4 tangent;

		assimp_plain_vtx() {}
		assimp_plain_vtx( const vec3& v, const vec2& t, const vec3& n, const vec4& g )
		{
			position = v;
			texcoord = t;
			normal   = n;
			tangent  = g;
		}
	};

	struct assimp_skinned_vtx 
	{
		vec3  position;
		vec3  normal;
		vec2  texcoord;
		vec4  tangent;
		ivec4 boneindex;
		vec4  boneweight;

		assimp_skinned_vtx() {}
		assimp_skinned_vtx( const vec3& v, const vec2& t, const vec3& n, const vec4& g )
		{
			position = v;
			texcoord = t;
			normal   = n;
			tangent  = g;
		}
	};

}

#endif // NV_ASSIMP_LOADER_HH
